<%
   traces = { "Application Trace" => @application_trace,
              "Framework Trace"   => @framework_trace,
              "Full Trace"        => @full_trace }
   names = traces.keys
%>

<p><code>Rails.root: <%= defined?(Rails) && Rails.respond_to?(:root) ? Rails.root : "unset" %></code></p>

<div id="traces">
  <% names.each do |name| %>
    <%
      show = "show('#{name.gsub(/\s/, '-')}');"
      hide = (names - [name]).collect {|hide_name| "hide('#{hide_name.gsub(/\s/, '-')}');"}
    %>
    <a href="#" onclick="<%= hide.join %><%= show %>; return false;"><%= name %></a> <%= '|' unless names.last == name %>
  <% end %>

  <% traces.each do |name, trace| %>
    <div id="<%= name.gsub(/\s/, '-') %>" style="display: <%= (name == "Application Trace") ? 'block' : 'none' %>;">
      <pre><code><%= trace.join "\n" %></code></pre>
    </div>
  <% end %>
</div>
